@@ -257,6 +257,144 @@ atomic_exchange_and_add (volatile long long *at, long long x,
 
 
 
+/// Atomic version of:  r = *at, *at &= x, return r
+/// For each of several architectures.
+inline int
+atomic_exchange_and_and (volatile int *at, int x,
+                        memory_order order = memory_order_seq_cst)
+{
+#ifdef NOTHREADS
+    int r = *at;  *at &= x;  return r;
+#elif defined(OIIO_USE_GCC_NEW_ATOMICS)
+    return __atomic_fetch_and (at, x, order);
+#elif defined(USE_GCC_ATOMICS)
+    return __sync_fetch_and_and ((int *)at, x);
+#elif defined(_MSC_VER)
+    // Windows
+    return _InterlockedAnd ((volatile LONG *)at, x);
+#else
+#   error No atomics on this platform.
+#endif
+}
+
+
+
+inline long long
+atomic_exchange_and_and (volatile long long *at, long long x,
+                        memory_order order = memory_order_seq_cst)
+{
+#ifdef NOTHREADS
+    long long r = *at;  *at &= x;  return r;
+#elif defined(OIIO_USE_GCC_NEW_ATOMICS)
+    return __atomic_fetch_and (at, x, order);
+#elif defined(USE_GCC_ATOMICS)
+    return __sync_fetch_and_and (at, x);
+#elif defined(_MSC_VER)
+    // Windows
+#  if defined(_WIN64)
+    return _InterlockedAnd64 ((volatile LONGLONG *)at, x);
+#  else
+    return InterlockedAnd64 ((volatile LONGLONG *)at, x);
+#  endif
+#else
+#   error No atomics on this platform.
+#endif
+}
+
+
+
+/// Atomic version of:  r = *at, *at |= x, return r
+/// For each of several architectures.
+inline int
+atomic_exchange_and_or (volatile int *at, int x,
+                        memory_order order = memory_order_seq_cst)
+{
+#ifdef NOTHREADS
+    int r = *at;  *at |= x;  return r;
+#elif defined(OIIO_USE_GCC_NEW_ATOMICS)
+    return __atomic_fetch_or (at, x, order);
+#elif defined(USE_GCC_ATOMICS)
+    return __sync_fetch_and_or ((int *)at, x);
+#elif defined(_MSC_VER)
+    // Windows
+    return _InterlockedOr ((volatile LONG *)at, x);
+#else
+#   error No atomics on this platform.
+#endif
+}
+
+
+
+inline long long
+atomic_exchange_and_or (volatile long long *at, long long x,
+                        memory_order order = memory_order_seq_cst)
+{
+#ifdef NOTHREADS
+    long long r = *at;  *at |= x;  return r;
+#elif defined(OIIO_USE_GCC_NEW_ATOMICS)
+    return __atomic_fetch_or (at, x, order);
+#elif defined(USE_GCC_ATOMICS)
+    return __sync_fetch_and_or (at, x);
+#elif defined(_MSC_VER)
+    // Windows
+#  if defined(_WIN64)
+    return _InterlockedOr64 ((volatile LONGLONG *)at, x);
+#  else
+    return InterlockedOr64 ((volatile LONGLONG *)at, x);
+#  endif
+#else
+#   error No atomics on this platform.
+#endif
+}
+
+
+
+/// Atomic version of:  r = *at, *at ^= x, return r
+/// For each of several architectures.
+inline int
+atomic_exchange_and_xor (volatile int *at, int x,
+                         memory_order order = memory_order_seq_cst)
+{
+#ifdef NOTHREADS
+    int r = *at;  *at ^= x;  return r;
+#elif defined(OIIO_USE_GCC_NEW_ATOMICS)
+    return __atomic_fetch_xor (at, x, order);
+#elif defined(USE_GCC_ATOMICS)
+    return __sync_fetch_and_xor ((int *)at, x);
+#elif defined(_MSC_VER)
+    // Windows
+    return _InterlockedXor ((volatile LONG *)at, x);
+#else
+#   error No atomics on this platform.
+#endif
+}
+
+
+
+inline long long
+atomic_exchange_and_xor (volatile long long *at, long long x,
+                         memory_order order = memory_order_seq_cst)
+{
+#ifdef NOTHREADS
+    long long r = *at;  *at ^= x;  return r;
+#elif defined(OIIO_USE_GCC_NEW_ATOMICS)
+    return __atomic_fetch_xor (at, x, order);
+#elif defined(USE_GCC_ATOMICS)
+    return __sync_fetch_and_xor (at, x);
+#elif defined(_MSC_VER)
+    // Windows
+#  if defined(_WIN64)
+    return _InterlockedXor64 ((volatile LONGLONG *)at, x);
+#  else
+    return InterlockedXor64 ((volatile LONGLONG *)at, x);
+#  endif
+#else
+#   error No atomics on this platform.
+#endif
+}
+
+
+
 /// Atomic version of:
 ///    if (*at == compareval) {
 ///        *at = newval;  return true;
@@ -508,6 +646,18 @@ public:
     T fetch_sub (T x, memory_order order = memory_order_seq_cst) {
         return atomic_exchange_and_add (&m_val, -x, order);
     }
+    /// Atomic fetch-and-and: bitwise and with x and return the old value.
+    T fetch_and (T x, memory_order order = memory_order_seq_cst) {
+        return atomic_exchange_and_and (&m_val, x, order);
+    }
+    /// Atomic fetch-and-or: bitwise or with x and return the old value.
+    T fetch_or (T x, memory_order order = memory_order_seq_cst) {
+        return atomic_exchange_and_or (&m_val, x, order);
+    }
+    /// Atomic fetch-and-xor: bitwise xor with x and return the old value.
+    T fetch_xor (T x, memory_order order = memory_order_seq_cst) {
+        return atomic_exchange_and_xor (&m_val, x, order);
+    }
 
     /// Assign new value.
     ///
@@ -537,6 +687,18 @@ public:
     ///
     T operator-= (T x) { return fetch_sub(x) - x; }
 
+    /// Logical and, return the new result
+    ///
+    T operator&= (T x) { return fetch_and(x) & x; }
+
+    /// Logical or, return the new result
+    ///
+    T operator|= (T x) { return fetch_or(x) | x; }
+
+    /// Logical xor, return the new result
+    ///
+    T operator^= (T x) { return fetch_xor(x) ^ x; }
+
     bool bool_compare_and_swap (T compareval, T newval) {
         return atomic_compare_and_exchange (&m_val, compareval, newval);
     }
@@ -834,6 +996,40 @@ typedef spin_rw_mutex::write_lock_guard spin_rw_write_lock;
 
 
 
+/// Mutex pool. Sometimes, we have lots of objects that need to be
+/// individually locked for thread safety, but two separate objects don't
+/// need to lock against each other. If there are many more objects than
+/// threads, it's wasteful for each object to contain its own mutex. So a
+/// solution is to make a mutex_pool -- a collection of several mutexes.
+/// Each object uses a hash to choose a consistent mutex for itself, but
+/// which will be unlikely to be locked simultaneously by different object.
+/// Semantically, it looks rather like an associative array of mutexes. We
+/// also ensure that the mutexes are all on different cache lines, to ensure
+/// that they don't exhibit false sharing. Try to choose Bins larger than
+/// the expected number of threads that will be simultaneously locking
+/// mutexes.
+template<class Mutex, class Key, class Hash, size_t Bins=16>
+class mutex_pool
+{
+public:
+    mutex_pool () { }
+    Mutex& operator[] (const Key &key) {
+        return m_mutex[m_hash(key) % Bins].m;
+    }
+private:
+    // Helper type -- force cache line alignment. This should make an array
+    // of these also have padding so that each individual mutex is aligned
+    // to its own cache line, thus eliminating any "false sharing."
+    struct AlignedMutex {
+        OIIO_CACHE_ALIGN Mutex m;
+    };
+
+    AlignedMutex m_mutex[Bins];
+    Hash m_hash;
+};
+
+
+
 /// Simple thread group class. This is just as good as boost::thread_group,
 /// for the limited functionality that we use.
 class thread_group {
