@@ -29,7 +29,6 @@
 */
 
 #include <string>
-#include <boost/scoped_ptr.hpp>
 
 #include "OpenImageIO/dassert.h"
 #include "OpenImageIO/typedesc.h"
@@ -41,8 +40,7 @@
 #include "OpenImageIO/texture.h"
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 
 namespace {  // anonymous
@@ -57,7 +55,8 @@ static int   default_samples = 1;
 static const ustring wrap_type_name[] = {
     // MUST match the order of TextureOptions::Wrap
     ustring("default"), ustring("black"), ustring("clamp"),
-    ustring("periodic"), ustring("mirror"),
+    ustring("periodic"), ustring("mirror"), ustring("periodic_pow2"),
+    ustring("periodic_sharedborder"),
     ustring()
 };
 
@@ -68,7 +67,7 @@ static const ustring wrap_type_name[] = {
 /// Special private ctr that makes a canonical default TextureOptions.
 /// For use internal to libtexture.  Users, don't call this!
 TextureOptions::TextureOptions ()
-    : firstchannel(0), nchannels(1), subimage(0),
+    : firstchannel(0), subimage(0),
       swrap(TextureOptions::WrapDefault), twrap(TextureOptions::WrapDefault),
       mipmode(TextureOptions::MipModeDefault),
       interpmode(TextureOptions::InterpSmartBicubic),
@@ -80,18 +79,15 @@ TextureOptions::TextureOptions ()
       fill(default_fill),
       missingcolor(NULL),
       samples(default_samples),
-      dresultds(NULL), dresultdt(NULL),
       rwrap(TextureOptions::WrapDefault),
-      rblur(default_blur), rwidth(default_width),
-      dresultdr(NULL),
-      swrap_func(NULL), twrap_func(NULL), rwrap_func(NULL)
+      rblur(default_blur), rwidth(default_width)
 {
 }
 
 
 
 TextureOptions::TextureOptions (const TextureOpt &opt)
-    : firstchannel(opt.firstchannel), nchannels(opt.nchannels),
+    : firstchannel(opt.firstchannel),
       subimage(opt.subimage), subimagename(opt.subimagename),
       swrap((Wrap)opt.swrap), twrap((Wrap)opt.twrap),
       mipmode((MipMode)opt.mipmode),
@@ -105,18 +101,15 @@ TextureOptions::TextureOptions (const TextureOpt &opt)
       fill((float *)&opt.fill),
       missingcolor((void *)opt.missingcolor),
       samples((int *)&opt.samples),
-      dresultds((float *)opt.dresultds), dresultdt((float *)opt.dresultdt),
       rwrap((Wrap)opt.rwrap), rblur((float *)&opt.rblur),
-      rwidth((float *)&opt.rwidth),
-      dresultdr((float *)opt.dresultdr),
-      swrap_func(NULL), twrap_func(NULL), rwrap_func(NULL)
+      rwidth((float *)&opt.rwidth)
 {
 }
 
 
 
 TextureOpt::TextureOpt (const TextureOptions &opt, int index)
-    : nchannels(opt.nchannels), firstchannel(opt.firstchannel),
+    : firstchannel(opt.firstchannel),
       subimage(opt.subimage), subimagename(opt.subimagename),
       swrap((Wrap)opt.swrap), twrap((Wrap)opt.twrap),
       mipmode((MipMode)opt.mipmode),
@@ -127,14 +120,11 @@ TextureOpt::TextureOpt (const TextureOptions &opt, int index)
       swidth(opt.swidth[index]), twidth(opt.twidth[index]),
       fill(opt.fill[index]),
       missingcolor(opt.missingcolor.ptr() ? &opt.missingcolor[index] : NULL),
-      dresultds(opt.dresultds), dresultdt(opt.dresultdt),
       time(opt.time[index]),
       bias(opt.bias[index]),
       samples(opt.samples[index]),
       rwrap((Wrap)opt.rwrap),
       rblur(opt.rblur[index]), rwidth(opt.rwidth[index]),
-      dresultdr(opt.dresultdr),
-      swrap_func(NULL), twrap_func(NULL), rwrap_func(NULL),
       envlayout(0)
 {
 }
@@ -181,5 +171,4 @@ TextureOpt::parse_wrapmodes (const char *wrapmodes,
     twrapcode = decode_wrapmode (twrap);
 }
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
