@@ -37,10 +37,9 @@
 #define OPENIMAGEIO_TEXTURE_PVT_H
 
 #include "OpenImageIO/texture.h"
+#include "OpenImageIO/simd.h"
 
-
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 class ImageCache;
 class Filter1D;
@@ -83,24 +82,24 @@ public:
         return attribute (name, TypeDesc::STRING, &s);
     }
 
-    virtual bool getattribute (string_view name, TypeDesc type, void *val);
-    virtual bool getattribute (string_view name, int &val) {
+    virtual bool getattribute (string_view name, TypeDesc type, void *val) const;
+    virtual bool getattribute (string_view name, int &val) const {
         return getattribute (name, TypeDesc::INT, &val);
     }
-    virtual bool getattribute (string_view name, float &val) {
+    virtual bool getattribute (string_view name, float &val) const {
         return getattribute (name, TypeDesc::FLOAT, &val);
     }
-    virtual bool getattribute (string_view name, double &val) {
+    virtual bool getattribute (string_view name, double &val) const {
         float f;
         bool ok = getattribute (name, TypeDesc::FLOAT, &f);
         if (ok)
             val = f;
         return ok;
     }
-    virtual bool getattribute (string_view name, char **val) {
+    virtual bool getattribute (string_view name, char **val) const {
         return getattribute (name, TypeDesc::STRING, val);
     }
-    virtual bool getattribute (string_view name, std::string &val) {
+    virtual bool getattribute (string_view name, std::string &val) const {
         const char *s;
         bool ok = getattribute (name, TypeDesc::STRING, &s);
         if (ok)
@@ -109,8 +108,6 @@ public:
     }
 
 
-    /// Close everything, free resources, start from scratch.
-    ///
     virtual void clear () { }
 
     // Retrieve options
@@ -118,139 +115,183 @@ public:
         result = m_Mc2w;
     }
 
-    virtual Perthread *get_perthread_info (void) {
-        return (Perthread *)m_imagecache->get_perthread_info ();
+    virtual Perthread *get_perthread_info (Perthread *thread_info = NULL) {
+        return (Perthread *)m_imagecache->get_perthread_info ((ImageCachePerThreadInfo *)thread_info);
+    }
+    virtual Perthread *create_thread_info () {
+        ASSERT (m_imagecache);
+        return (Perthread *)m_imagecache->create_thread_info ();
+    }
+    virtual void destroy_thread_info (Perthread *threadinfo) {
+        ASSERT (m_imagecache);
+        m_imagecache->destroy_thread_info ((ImageCachePerThreadInfo *)threadinfo);
     }
 
     virtual TextureHandle *get_texture_handle (ustring filename,
                                                Perthread *thread) {
-        PerThreadInfo *thread_info = thread ? (PerThreadInfo *)thread
+        PerThreadInfo *thread_info = thread ? ((PerThreadInfo *)thread)
                                        : m_imagecache->get_perthread_info ();
         return (TextureHandle *) find_texturefile (filename, thread_info);
     }
 
-    /// Filtered 2D texture lookup for a single point, no runflags.
-    ///
-    virtual bool texture (TextureHandle *texture_handle, Perthread *thread_info,
-                          TextureOpt &options,
-                          float s, float t,
-                          float dsdx, float dtdx, float dsdy, float dtdy,
-                          float *result);
-
-
-    /// Filtered 2D texture lookup for a single point, no runflags.
-    ///
-    virtual bool texture (ustring filename, TextureOpt &options,
-                          float s, float t,
-                          float dsdx, float dtdx, float dsdy, float dtdy,
-                          float *result);
-
-    /// Filtered 2D texture lookup for a single point, no runflags.
-    ///
-    virtual bool texture (ustring filename, TextureOptions &options,
-                  float s, float t,
-                  float dsdx, float dtdx, float dsdy, float dtdy,
-                  float *result) {
-        Runflag rf = RunFlagOn;
-        return texture (filename, options, &rf, 0, 1, s, t,
-                        dsdx, dtdx, dsdy, dtdy, result);
+    virtual bool good (TextureHandle *texture_handle) {
+        return texture_handle && ! ((TextureFile *)texture_handle)->broken();
     }
 
-    /// Retrieve a 2D texture lookup at many points at once.
-    ///
+    virtual bool texture (ustring filename, TextureOpt &options,
+                          float s, float t, float dsdx, float dtdx,
+                          float dsdy, float dtdy,
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL);
+    virtual bool texture (TextureHandle *texture_handle,
+                          Perthread *thread_info, TextureOpt &options,
+                          float s, float t, float dsdx, float dtdx,
+                          float dsdy, float dtdy,
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL);
     virtual bool texture (ustring filename, TextureOptions &options,
                           Runflag *runflags, int beginactive, int endactive,
                           VaryingRef<float> s, VaryingRef<float> t,
                           VaryingRef<float> dsdx, VaryingRef<float> dtdx,
                           VaryingRef<float> dsdy, VaryingRef<float> dtdy,
-                          float *result);
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL);
+    virtual bool texture (TextureHandle *texture_handle,
+                          Perthread *thread_info, TextureOptions &options,
+                          Runflag *runflags, int beginactive, int endactive,
+                          VaryingRef<float> s, VaryingRef<float> t,
+                          VaryingRef<float> dsdx, VaryingRef<float> dtdx,
+                          VaryingRef<float> dsdy, VaryingRef<float> dtdy,
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL);
+
 
-    /// Retrieve a 3D texture lookup at a single point.
-    ///
     virtual bool texture3d (ustring filename, TextureOpt &options,
                             const Imath::V3f &P, const Imath::V3f &dPdx,
                             const Imath::V3f &dPdy, const Imath::V3f &dPdz,
-                            float *result);
-
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL);
     virtual bool texture3d (TextureHandle *texture_handle,
                             Perthread *thread_info, TextureOpt &options,
                             const Imath::V3f &P, const Imath::V3f &dPdx,
                             const Imath::V3f &dPdy, const Imath::V3f &dPdz,
-                            float *result);
-
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL);
     virtual bool texture3d (ustring filename, TextureOptions &options,
                             Runflag *runflags, int beginactive, int endactive,
                             VaryingRef<Imath::V3f> P,
                             VaryingRef<Imath::V3f> dPdx,
                             VaryingRef<Imath::V3f> dPdy,
                             VaryingRef<Imath::V3f> dPdz,
-                            float *result);
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL);
+    virtual bool texture3d (TextureHandle *texture_handle,
+                            Perthread *thread_info, TextureOptions &options,
+                            Runflag *runflags, int beginactive, int endactive,
+                            VaryingRef<Imath::V3f> P,
+                            VaryingRef<Imath::V3f> dPdx,
+                            VaryingRef<Imath::V3f> dPdy,
+                            VaryingRef<Imath::V3f> dPdz,
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL);
 
-    /// Retrieve a shadow lookup for a single position P.
-    ///
     virtual bool shadow (ustring filename, TextureOpt &options,
                          const Imath::V3f &P, const Imath::V3f &dPdx,
-                         const Imath::V3f &dPdy, float *result) {
+                         const Imath::V3f &dPdy, float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) {
         return false;
     }
-
     virtual bool shadow (TextureHandle *texture_handle, Perthread *thread_info,
                          TextureOpt &options,
                          const Imath::V3f &P, const Imath::V3f &dPdx,
-                         const Imath::V3f &dPdy, float *result) {
+                         const Imath::V3f &dPdy, float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) {
         return false;
     }
-
-    /// Retrieve a shadow lookup for position P at many points at once.
-    ///
     virtual bool shadow (ustring filename, TextureOptions &options,
                          Runflag *runflags, int beginactive, int endactive,
                          VaryingRef<Imath::V3f> P,
                          VaryingRef<Imath::V3f> dPdx,
                          VaryingRef<Imath::V3f> dPdy,
-                         float *result) {
+                         float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) {
+        return false;
+    }
+    virtual bool shadow (TextureHandle *texture_handle, Perthread *thread_info,
+                         TextureOptions &options,
+                         Runflag *runflags, int beginactive, int endactive,
+                         VaryingRef<Imath::V3f> P,
+                         VaryingRef<Imath::V3f> dPdx,
+                         VaryingRef<Imath::V3f> dPdy,
+                         float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) {
         return false;
     }
 
-    /// Retrieve an environment map lookup for direction R.
-    ///
-    virtual bool environment (ustring filename, TextureOpt &opt,
-                              const Imath::V3f &R, const Imath::V3f &dRdx,
-                              const Imath::V3f &dRdy, float *result);
 
+    virtual bool environment (ustring filename, TextureOpt &options,
+                              const Imath::V3f &R, const Imath::V3f &dRdx,
+                              const Imath::V3f &dRdy, int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL);
     virtual bool environment (TextureHandle *texture_handle,
                               Perthread *thread_info, TextureOpt &options,
                               const Imath::V3f &R, const Imath::V3f &dRdx,
-                              const Imath::V3f &dRdy, float *result);
-
-    /// Retrieve an environment map lookup for direction R, for many
-    /// points at once.
+                              const Imath::V3f &dRdy, int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL);
     virtual bool environment (ustring filename, TextureOptions &options,
                               Runflag *runflags, int beginactive, int endactive,
                               VaryingRef<Imath::V3f> R,
                               VaryingRef<Imath::V3f> dRdx,
                               VaryingRef<Imath::V3f> dRdy,
-                              float *result);
+                              int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL);
+    virtual bool environment (TextureHandle *texture_handle,
+                              Perthread *thread_info, TextureOptions &options,
+                              Runflag *runflags, int beginactive, int endactive,
+                              VaryingRef<Imath::V3f> R,
+                              VaryingRef<Imath::V3f> dRdx,
+                              VaryingRef<Imath::V3f> dRdy,
+                              int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL);
 
-    /// Given possibly-relative 'filename', resolve it using the search
-    /// path rules and return the full resolved filename.
     virtual std::string resolve_filename (const std::string &filename) const;
 
-    /// Get information about the given texture.
-    ///
     virtual bool get_texture_info (ustring filename, int subimage,
                            ustring dataname, TypeDesc datatype, void *data);
+    virtual bool get_texture_info (TextureHandle *texture_handle,
+                           Perthread *thread_info, int subimage,
+                           ustring dataname, TypeDesc datatype, void *data);
+    virtual bool get_texture_info (TextureHandle *texture_handle, int subimage,
+                           ustring dataname, TypeDesc datatype, void *data) {
+        return get_texture_info (texture_handle, NULL, subimage,
+                                 dataname, datatype, data);
+    }
 
     virtual bool get_imagespec (ustring filename, int subimage,
                                 ImageSpec &spec);
+    virtual bool get_imagespec (TextureHandle *texture_handle,
+                                Perthread *thread_info, int subimage,
+                                ImageSpec &spec);
 
     virtual const ImageSpec *imagespec (ustring filename, int subimage=0);
+    virtual const ImageSpec *imagespec (TextureHandle *texture_handle,
+                                        Perthread *thread_info=NULL,
+                                        int subimage=0);
 
-    /// Retrieve a rectangle of raw unfiltered texels.
-    ///
     virtual bool get_texels (ustring filename, TextureOpt &options,
                              int miplevel, int xbegin, int xend,
                              int ybegin, int yend, int zbegin, int zend,
+                             int chbegin, int chend,
+                             TypeDesc format, void *result);
+    virtual bool get_texels (TextureHandle *texture_handle,
+                             Perthread *thread_info, TextureOpt &options,
+                             int miplevel, int xbegin, int xend,
+                             int ybegin, int yend, int zbegin, int zend,
+                             int chbegin, int chend,
                              TypeDesc format, void *result);
 
     virtual std::string geterror () const;
@@ -262,6 +303,8 @@ public:
 
     void operator delete (void *todel) { ::delete ((char *)todel); }
 
+    typedef bool (*wrap_impl) (int &coord, int origin, int width);
+
 private:
     typedef ImageCacheTileRef TileRef;
     typedef ImageCachePerThreadInfo PerThreadInfo;
@@ -271,9 +314,15 @@ private:
     /// Find the TextureFile record for the named texture, or NULL if no
     /// such file can be found.
     TextureFile *find_texturefile (ustring filename, PerThreadInfo *thread_info) {
-        TextureFile *texturefile = m_imagecache->find_file (filename, thread_info);
-        if (!texturefile || texturefile->broken())
-            error ("%s", m_imagecache->geterror().c_str());
+        return m_imagecache->find_file (filename, thread_info);
+    }
+    TextureFile *verify_texturefile (TextureFile *texturefile,
+                                     PerThreadInfo *thread_info) {
+        texturefile = m_imagecache->verify_file (texturefile, thread_info);
+        if (!texturefile || texturefile->broken()) {
+            std::string err = m_imagecache->geterror();
+            error ("%s", err.size() ? err.c_str() : "(unknown error)");
+        }
         return texturefile;
     }
 
@@ -291,94 +340,115 @@ private:
 
     // Define a prototype of a member function pointer for texture
     // lookups.
+    // If simd is nonzero, it's guaranteed that all float* inputs and
+    // outputs are padded to length 'simd' and aligned to a simd*4-byte
+    // boundary (for example, 4 for SSE). This means that the functions can
+    // behave AS IF the number of channels being retrieved is simd, and any
+    // extra values returned will be discarded by the caller.
     typedef bool (TextureSystemImpl::*texture_lookup_prototype)
             (TextureFile &texfile, PerThreadInfo *thread_info,
              TextureOpt &options,
+             int nchannels_result, int actualchannels,
              float _s, float _t,
              float _dsdx, float _dtdx,
              float _dsdy, float _dtdy,
-             float *result);
+             float *result, float *dresultds, float *resultdt);
 
     /// Look up texture from just ONE point
     ///
     bool texture_lookup (TextureFile &texfile, PerThreadInfo *thread_info, 
                          TextureOpt &options,
+                         int nchannels_result, int actualchannels,
                          float _s, float _t,
                          float _dsdx, float _dtdx,
                          float _dsdy, float _dtdy,
-                         float *result);
+                         float *result, float *dresultds, float *resultdt);
     
     bool texture_lookup_nomip (TextureFile &texfile, 
                          PerThreadInfo *thread_info, 
                          TextureOpt &options,
+                         int nchannels_result, int actualchannels,
                          float _s, float _t,
                          float _dsdx, float _dtdx,
                          float _dsdy, float _dtdy,
-                         float *result);
+                         float *result, float *dresultds, float *resultdt);
     
     bool texture_lookup_trilinear_mipmap (TextureFile &texfile,
                          PerThreadInfo *thread_info, 
                          TextureOpt &options,
+                         int nchannels_result, int actualchannels,
                          float _s, float _t,
                          float _dsdx, float _dtdx,
                          float _dsdy, float _dtdy,
-                         float *result);
+                         float *result, float *dresultds, float *resultdt);
     
-    typedef bool (TextureSystemImpl::*accum_prototype)
-                              (float s, float t, int level,
-                               TextureFile &texturefile,
-                               PerThreadInfo *thread_info,
-                               TextureOpt &options,
-                               float weight, float *accum,
-                               float *daccumds, float *daccumdt);
-
-    bool accum_sample_closest (float s, float t, int level,
-                               TextureFile &texturefile,
-                               PerThreadInfo *thread_info,
-                               TextureOpt &options,
-                               float weight, float *accum,
-                               float *daccumds, float *daccumdt);
-
-    bool accum_sample_bilinear (float s, float t, int level,
-                                TextureFile &texturefile,
-                                PerThreadInfo *thread_info,
-                                TextureOpt &options,
-                                float weight, float *accum,
-                                float *daccumds, float *daccumdt);
-
-    bool accum_sample_bicubic (float s, float t, int level,
-                               TextureFile &texturefile,
-                               PerThreadInfo *thread_info,
-                               TextureOpt &options,
-                               float weight, float *accum,
-                               float *daccumds, float *daccumdt);
+    // For the samplers, it's guaranteed that all float* inputs and outputs
+    // are padded to length 'simd' and aligned to a simd*4-byte boundary
+    // (for example, 4 for SSE). This means that the functions can behave AS
+    // IF the number of channels being retrieved is simd, and any extra
+    // values returned will be discarded by the caller.
+    typedef bool (TextureSystemImpl::*sampler_prototype)
+                         (int nsamples, const float *s, const float *t,
+                          int level, TextureFile &texturefile,
+                          PerThreadInfo *thread_info, TextureOpt &options,
+                          int nchannels_result, int actualchannels,
+                          const float *weight, simd::float4 *accum,
+                          simd::float4 *daccumds, simd::float4 *daccumdt);
+    bool sample_closest  (int nsamples, const float *s, const float *t,
+                          int level, TextureFile &texturefile,
+                          PerThreadInfo *thread_info, TextureOpt &options,
+                          int nchannels_result, int actualchannels,
+                          const float *weight, simd::float4 *accum,
+                          simd::float4 *daccumds, simd::float4 *daccumdt);
+    bool sample_bilinear (int nsamples, const float *s, const float *t,
+                          int level, TextureFile &texturefile,
+                          PerThreadInfo *thread_info, TextureOpt &options,
+                          int nchannels_result, int actualchannels,
+                          const float *weight, simd::float4 *accum,
+                          simd::float4 *daccumds, simd::float4 *daccumdt);
+    bool sample_bicubic  (int nsamples, const float *s, const float *t,
+                          int level, TextureFile &texturefile,
+                          PerThreadInfo *thread_info, TextureOpt &options,
+                          int nchannels_result, int actualchannels,
+                          const float *weight, simd::float4 *accum,
+                          simd::float4 *daccumds, simd::float4 *daccumdt);
 
     // Define a prototype of a member function pointer for texture3d
     // lookups.
     typedef bool (TextureSystemImpl::*texture3d_lookup_prototype)
             (TextureFile &texfile, PerThreadInfo *thread_info,
              TextureOpt &options,
+             int nchannels_result, int actualchannels,
              const Imath::V3f &P, const Imath::V3f &dPdx,
              const Imath::V3f &dPdy, const Imath::V3f &dPdz,
-             float *result);
+             float *result, float *dresultds,
+             float *dresultdt, float *dresultdr);
     bool texture3d_lookup_nomip (TextureFile &texfile,
                                  PerThreadInfo *thread_info, 
                                  TextureOpt &options,
+                                 int nchannels_result, int actualchannels,
                                  const Imath::V3f &P, const Imath::V3f &dPdx,
                                  const Imath::V3f &dPdy, const Imath::V3f &dPdz,
-                                 float *result);
+                                 float *result, float *dresultds,
+                                 float *dresultdt, float *dresultdr);
     typedef bool (TextureSystemImpl::*accum3d_prototype)
                         (const Imath::V3f &P, int level,
                          TextureFile &texturefile, PerThreadInfo *thread_info,
-                         TextureOpt &options, float weight, float *accum,
+                         TextureOpt &options,
+                         int nchannels_result, int actualchannels,
+                         float weight, float *accum,
                          float *daccumds, float *daccumdt, float *daccumdr);
     bool accum3d_sample_closest (const Imath::V3f &P, int level,
                 TextureFile &texturefile, PerThreadInfo *thread_info,
-                TextureOpt &options, float weight, float *accum,
+                TextureOpt &options,
+                int nchannels_result, int actualchannels,
+                float weight, float *accum,
                 float *daccumds, float *daccumdt, float *daccumdr);
     bool accum3d_sample_bilinear (const Imath::V3f &P, int level,
                 TextureFile &texturefile, PerThreadInfo *thread_info,
-                TextureOpt &options, float weight, float *accum,
+                TextureOpt &options,
+                int nchannels_result, int actualchannels,
+                float weight, float *accum,
                 float *daccumds, float *daccumdt, float *daccumdr);
 
     /// Helper function to calculate the anisotropic aspect ratio from
@@ -401,10 +471,14 @@ private:
 
     /// Called when the requested texture is missing, fills in the
     /// results.
-    bool missing_texture (TextureOpt &options, float *result);
+    bool missing_texture (TextureOpt &options, int nchannels, float *result,
+                          float *dresultds, float *dresultdt,
+                          float *dresultdr=NULL);
 
     /// Handle gray-to-RGB promotion.
-    void fill_gray_channels (const ImageSpec &spec, TextureOpt &options, float *result);
+    void fill_gray_channels (const ImageSpec &spec, int nchannels,
+                             float *result, float *dresultds, float *dresultdt,
+                             float *dresultdr=NULL);
 
     static bool wrap_periodic_sharedborder (int &coord, int origin, int width);
     static const wrap_impl wrap_functions[];
@@ -449,6 +523,8 @@ private:
     Imath::M44f m_Mw2c;          ///< world-to-"common" matrix
     Imath::M44f m_Mc2w;          ///< common-to-world matrix
     bool m_gray_to_rgb;          ///< automatically copy gray to rgb channels?
+    int m_max_tile_channels;     ///< narrow tile ID channel range when
+                                 ///<   the file has more channels
     /// Saved error string, per-thread
     ///
     mutable thread_specific_ptr< std::string > m_errormessage;
@@ -479,7 +555,7 @@ TextureSystemImpl::anisotropic_aspect (float &majorlength, float &minorlength,
         //    aliasing along the major axis.  You can't please everybody.
         if (options.conservative_filter) {
 #if 0
-            // Soltion (a) -- never alias by blurring more along minor axis
+            // Solution (a) -- never alias by blurring more along minor axis
             minorlength = majorlength / options.anisotropic;
 #else
             // Solution (c) -- this is our default, usually a nice balance.
@@ -532,7 +608,6 @@ TextureSystemImpl::st_to_texel (float s, float t, TextureFile &texturefile,
 
 }  // end namespace pvt
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
 
 #endif // OPENIMAGEIO_TEXTURE_PVT_H
