@@ -56,6 +56,11 @@
 
 OIIO_NAMESPACE_BEGIN
 
+#ifdef _MSC_VER
+    // fix for https://svn.boost.org/trac/boost/ticket/6320
+    const std::string dummy_path = "../dummy_path.txt";
+    const std::string dummy_extension = boost::filesystem::path(dummy_path).extension().string();
+#endif
 
 std::string
 Filesystem::filename (const std::string &filepath)
@@ -152,37 +157,56 @@ Filesystem::searchpath_split (const std::string &searchpath,
 
 
 std::string
-Filesystem::searchpath_find (const std::string &filename,
+Filesystem::searchpath_find (const std::string &filename_utf8,
                              const std::vector<std::string> &dirs,
                              bool testcwd, bool recursive)
 {
-    bool abs = Filesystem::path_is_absolute (filename);
+#ifdef _WIN32
+    const boost::filesystem::path filename (Strutil::utf8_to_utf16 (filename_utf8));
+#else
+    const boost::filesystem::path filename (filename_utf8);
+#endif
+    bool abs = filename.is_absolute();
 
     // If it's an absolute filename, or if we want to check "." first,
     // then start by checking filename outright.
     if (testcwd || abs) {
-        if (Filesystem::is_regular (filename))
-            return filename;
+        if (boost::filesystem::is_regular_file (filename))
+            return filename_utf8;
     }
 
     // Relative filename, not yet found -- try each directory in turn
-    BOOST_FOREACH (const std::string &d, dirs) {
+    BOOST_FOREACH (const std::string &d_utf8, dirs) {
         // std::cerr << "\tPath = '" << d << "'\n";
-        boost::filesystem::path f = d;
-        f /= filename;
-        // std::cerr << "\tTesting '" << f << "'\n";
-        if (Filesystem::is_regular (f.string())) {
-            // std::cerr << "Found '" << f << "'\n";
+#ifdef _WIN32
+        const boost::filesystem::path d(Strutil::utf8_to_utf16 (d_utf8));
+#else
+        const boost::filesystem::path d(d_utf8);
+#endif
+        boost::filesystem::path f = d / filename;
+        // std::cerr << "\tTesting '" << f.string() << "'\n";
+        if (boost::filesystem::is_regular_file (f)) {
+            // std::cerr << "Found '" << f.string() << "'\n";
+#ifdef _WIN32
+            return Strutil::utf16_to_utf8 (f.native());
+#else
             return f.string();
+#endif
         }
 
-        if (recursive && Filesystem::is_directory (d)) {
+        if (recursive && boost::filesystem::is_directory (d)) {
             std::vector<std::string> subdirs;
-            for (boost::filesystem::directory_iterator s(d); 
-                 s != boost::filesystem::directory_iterator();  ++s)
-                if (Filesystem::is_directory(s->path().string()))
+            boost::filesystem::directory_iterator end_iter;
+            for (boost::filesystem::directory_iterator s(d); s != end_iter; ++s) {
+                if (boost::filesystem::is_directory (s->status())) {
+#ifdef _WIN32
+                    subdirs.push_back (Strutil::utf16_to_utf8 (s->path().native()));
+#else
                     subdirs.push_back (s->path().string());
-            std::string found = searchpath_find (filename, subdirs, false, true);
+#endif
+                }
+            }
+            std::string found = searchpath_find (filename_utf8, subdirs, false, true);
             if (found.size())
                 return found;
         }
@@ -210,14 +234,24 @@ Filesystem::get_directory_entries (const std::string &dirname,
     }
 
     if (recursive) {
+#ifdef _WIN32
+        std::wstring wdirpath = Strutil::utf8_to_utf16 (dirpath.string());
+        for (boost::filesystem::recursive_directory_iterator s (wdirpath);
+#else
         for (boost::filesystem::recursive_directory_iterator s (dirpath);
+#endif
              s != boost::filesystem::recursive_directory_iterator();  ++s) {
             std::string file = s->path().string();
             if (!filter_regex.size() || boost::regex_search (file, re))
                 filenames.push_back (file);
         }
     } else {
+#ifdef _WIN32
+        std::wstring wdirpath = Strutil::utf8_to_utf16 (dirpath.string());
+        for (boost::filesystem::directory_iterator s (wdirpath);
+#else
         for (boost::filesystem::directory_iterator s (dirpath);
+#endif
              s != boost::filesystem::directory_iterator();  ++s) {
             std::string file = s->path().string();
             if (!filter_regex.size() || boost::regex_search (file, re))
@@ -628,6 +662,26 @@ Filesystem::read_text_file (string_view filename, std::string &str)
 
 
 
+/// Read the entire contents of the named file and place it in str,
+/// returning true on success, false on failure.
+size_t
+Filesystem::read_bytes (string_view path, void *buffer, size_t n, size_t pos)
+{
+    size_t ret = 0;
+    if (FILE *file = Filesystem::fopen (path, "rb")) {
+#ifdef _MSC_VER
+        _fseeki64 (file, __int64(pos), SEEK_SET);
+#else
+        fseeko (file, pos, SEEK_SET);
+#endif
+        ret = fread (buffer, 1, n, file);
+        fclose (file);
+    }
+    return ret;
+}
+
+
+
 std::time_t
 Filesystem::last_write_time (const std::string& path)
 {
@@ -663,6 +717,24 @@ Filesystem::last_write_time (const std::string& path, std::time_t time)
 
 
 
+uint64_t
+Filesystem::file_size (string_view path)
+{
+    try {
+#ifdef _WIN32
+        std::wstring wpath = Strutil::utf8_to_utf16 (path);
+        return boost::filesystem::file_size (wpath);
+#else
+        return boost::filesystem::file_size (path.str());
+#endif
+    } catch (...) {
+        // File doesn't exist
+        return 0;
+    }
+}
+
+
+
 void
 Filesystem::convert_native_arguments (int argc, const char *argv[])
 {
