@@ -118,7 +118,7 @@ const ImageSpec& ImageInputWrap::spec() const
     return m_input->spec();
 }
 
-bool ImageInputWrap::supports (const std::string &feature) const
+int ImageInputWrap::supports (const std::string &feature) const
 {
     return m_input->supports (feature);
 }
@@ -147,69 +147,33 @@ bool ImageInputWrap::seek_subimage(int subimage, int miplevel)
 
 
 
-const char *
-python_array_code (TypeDesc format)
-{
-    switch (format.basetype) {
-    case TypeDesc::UINT8 :  return "B";
-    case TypeDesc::INT8 :   return "b";
-    case TypeDesc::UINT16 : return "H";
-    case TypeDesc::INT16 :  return "h";
-    case TypeDesc::UINT32 : return "I";
-    case TypeDesc::INT32 :  return "i";
-    case TypeDesc::FLOAT :  return "f";
-    case TypeDesc::DOUBLE : return "d";
-    default :               return "f";   // Punt -- return float
-    }
-}
-
-
-
-object
-C_array_to_Python_array (const char *data, TypeDesc type, size_t size)
-{
-    // Construct a Python array, convert the buffer we read into a string
-    // and then into the array.
-    object arr_module(handle<>(PyImport_ImportModule("array")));
-    object array = arr_module.attr("array")(python_array_code(type));
-#if PY_MAJOR_VERSION >= 3
-    object string_py(handle<>(PyBytes_FromStringAndSize(data, size)));
-#else
-    object string_py(handle<>(PyString_FromStringAndSize(data, size)));
-#endif
-#if (PY_MAJOR_VERSION < 3) || (PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION < 2)
-    array.attr("fromstring")(string_py);
-#else
-    array.attr("frombytes")(string_py);
-#endif
-    return array;
-}
-
-
-
 // The read_image method is a bit different from the c++ interface. 
 // "function" is a function which takes a float, and the 
 // PyProgressCallback function is called automatically.
 object
-ImageInputWrap::read_image (TypeDesc format)
+ImageInputWrap::read_image (int chbegin, int chend, TypeDesc format)
 {
     // Allocate our own temp buffer and try to read the image into it.
     // If the read fails, return None.
     const ImageSpec &spec = m_input->spec();
-    if (format.basetype == TypeDesc::UNKNOWN)
-        format = spec.format;
-    size_t size = (size_t) spec.image_pixels() * spec.nchannels * format.size();
+    bool native = (format.basetype == TypeDesc::UNKNOWN);
+    if (chend < 0)
+        chend = spec.nchannels;
+    chend = clamp (chend, chbegin+1, spec.nchannels);
+    size_t nchans = size_t(chend - chbegin);
+    size_t pixelsize = native ? spec.pixel_bytes(chbegin, chend, native)
+                              : size_t(nchans * format.size());
+    size_t size = size_t(spec.image_pixels()) * pixelsize;
     char *data = new char[size];
     bool ok;
     {
         ScopedGILRelease gil;
-        ok = m_input->read_image(format, data);
+        ok = m_input->read_image (chbegin, chend, format, data);
     }
     if (! ok) {
         delete [] data;  // never mind
         return object(handle<>(Py_None));
     }
-
     object array = C_array_to_Python_array (data, format, size);
 
     // clean up and return the array handle
@@ -221,14 +185,31 @@ ImageInputWrap::read_image (TypeDesc format)
 object
 ImageInputWrap_read_image_bt (ImageInputWrap& in, TypeDesc::BASETYPE format)
 {
-    return in.read_image (format);
+    return in.read_image (0, -1, format);
+}
+
+
+object
+ImageInputWrap_read_image_bt_chans (ImageInputWrap& in, int chbegin, int chend,
+                              TypeDesc::BASETYPE format)
+{
+    return in.read_image (chbegin, chend, format);
 }
 
 
 object
 ImageInputWrap_read_image_default (ImageInputWrap& in)
 {
-    return in.read_image (TypeDesc::UNKNOWN);
+    return in.read_image (0, -1, TypeDesc::FLOAT);
+}
+
+
+
+object
+ImageInputWrap_read_image_default_chans (ImageInputWrap& in,
+                                         int chbegin, int chend)
+{
+    return in.read_image (chbegin, chend, TypeDesc::FLOAT);
 }
 
 
@@ -236,28 +217,8 @@ ImageInputWrap_read_image_default (ImageInputWrap& in)
 object
 ImageInputWrap::read_scanline (int y, int z, TypeDesc format)
 {
-    // Allocate our own temp buffer and try to read the scanline into it.
-    // If the read fails, return None.
     const ImageSpec &spec = m_input->spec();
-    if (format.basetype == TypeDesc::UNKNOWN)
-        format = spec.format;
-    size_t size = (size_t) spec.width * spec.nchannels * format.size();
-    char *data = new char[size];
-    bool ok;
-    {
-        ScopedGILRelease gil;
-        ok = m_input->read_scanline (y, z, format, data);
-    }
-    if (!ok) {
-        delete [] data;  // never mind
-        return object(handle<>(Py_None));
-    }
-
-    object array = C_array_to_Python_array (data, format, size);
-
-    // clean up and return the array handle
-    delete [] data;
-    return array;
+    return read_scanlines (y, y+1, z, 0, spec.nchannels, format);
 }
 
 
@@ -272,7 +233,7 @@ ImageInputWrap_read_scanline_bt (ImageInputWrap& in, int y, int z,
 object
 ImageInputWrap_read_scanline_default (ImageInputWrap& in, int y, int z)
 {
-    return in.read_scanline (y, z, TypeDesc::UNKNOWN);
+    return in.read_scanline (y, z, TypeDesc::FLOAT);
 }
 
 
@@ -285,11 +246,12 @@ ImageInputWrap::read_scanlines (int ybegin, int yend, int z,
     // If the read fails, return None.
     ASSERT (m_input);
     const ImageSpec &spec = m_input->spec();
-    if (format.basetype == TypeDesc::UNKNOWN)
-        format = spec.format;
+    bool native = (format.basetype == TypeDesc::UNKNOWN);
     chend = clamp (chend, chbegin+1, spec.nchannels);
-    int nchans = chend - chbegin;
-    size_t size = (size_t) spec.width * (yend-ybegin) * nchans * format.size();
+    size_t nchans = size_t(chend - chbegin);
+    size_t pixelsize = native ? spec.pixel_bytes(chbegin, chend, native)
+                              : size_t(nchans * format.size());
+    size_t size = size_t((yend-ybegin) * spec.width) * pixelsize;
     char *data = new char[size];
     bool ok;
     {
@@ -322,7 +284,7 @@ object
 ImageInputWrap_read_scanlines_default (ImageInputWrap& in, int ybegin, int yend,
                                        int z, int chbegin, int chend)
 {
-    return in.read_scanlines (ybegin, yend, z, chbegin, chend, TypeDesc::UNKNOWN);
+    return in.read_scanlines (ybegin, yend, z, chbegin, chend, TypeDesc::FLOAT);
 }
 
 
@@ -330,28 +292,9 @@ ImageInputWrap_read_scanlines_default (ImageInputWrap& in, int ybegin, int yend,
 object
 ImageInputWrap::read_tile (int x, int y, int z, TypeDesc format)
 {
-    // Allocate our own temp buffer and try to read the scanline into it.
-    // If the read fails, return None.
     const ImageSpec &spec = m_input->spec();
-    if (format.basetype == TypeDesc::UNKNOWN)
-        format = spec.format;
-    size_t size = (size_t) spec.tile_pixels() * spec.nchannels * format.size();
-    char *data = new char[size];
-    bool ok;
-    {
-        ScopedGILRelease gil;
-        ok = m_input->read_tile (x, y, z, format, data);
-    }
-    if (! ok) {
-        delete [] data;  // never mind
-        return object(handle<>(Py_None));
-    }
-
-    object array = C_array_to_Python_array (data, format, size);
-
-    // clean up and return the array handle
-    delete [] data;
-    return array;
+    return read_tiles (x, x+spec.tile_width, y, y+spec.tile_height,
+                       z, z+spec.tile_depth, 0, spec.nchannels, format);
 }
 
 
@@ -367,7 +310,7 @@ object
 ImageInputWrap_read_tile_default (ImageInputWrap& in,
                                   int x, int y, int z)
 {
-    return in.read_tile (x, y, z, TypeDesc::UNKNOWN);
+    return in.read_tile (x, y, z, TypeDesc::FLOAT);
 }
 
 
@@ -380,12 +323,14 @@ ImageInputWrap::read_tiles (int xbegin, int xend, int ybegin, int yend,
     // Allocate our own temp buffer and try to read the scanline into it.
     // If the read fails, return None.
     const ImageSpec &spec = m_input->spec();
-    if (format.basetype == TypeDesc::UNKNOWN)
-        format = spec.format;
+    bool native = (format.basetype == TypeDesc::UNKNOWN);
+    if (chend < 0)
+        chend = spec.nchannels;
     chend = clamp (chend, chbegin+1, spec.nchannels);
-    int nchans = chend - chbegin;
-    size_t size = (size_t) ((xend-xbegin) * (yend-ybegin) * 
-                            (zend-zbegin) * nchans * format.size());
+    size_t nchans = size_t(chend - chbegin);
+    size_t pixelsize = native ? spec.pixel_bytes(chbegin, chend, native)
+                              : size_t(nchans * format.size());
+    size_t size = size_t((xend-xbegin) * (yend-ybegin) * (zend-zbegin)) * pixelsize;
     char *data = new char[size];
     bool ok;
     {
@@ -423,7 +368,7 @@ ImageInputWrap_read_tiles_default (ImageInputWrap& in,
                                    int zbegin, int zend, int chbegin, int chend)
 {
     return in.read_tiles (xbegin, xend, ybegin, yend, zbegin, zend,
-                          chbegin, chend, TypeDesc::UNKNOWN);
+                          chbegin, chend, TypeDesc::FLOAT);
 }
 
 
@@ -529,7 +474,9 @@ void declare_imageinput()
         .def("read_tiles",       &ImageInputWrap_read_tiles_default)
         .def("read_image",       &ImageInputWrap::read_image)
         .def("read_image",       &ImageInputWrap_read_image_bt)
+        .def("read_image",       &ImageInputWrap_read_image_bt_chans)
         .def("read_image",       &ImageInputWrap_read_image_default)
+        .def("read_image",       &ImageInputWrap_read_image_default_chans)
         .def("read_native_deep_scanlines", &ImageInputWrap::read_native_deep_scanlines)
         .def("read_native_deep_tiles",     &ImageInputWrap::read_native_deep_tiles)
         .def("read_native_deep_image",     &ImageInputWrap::read_native_deep_image)
