@@ -1,5 +1,6 @@
 #!/usr/bin/env python 
 
+import math
 import OpenImageIO as oiio
 from OpenImageIO import ImageBuf, ImageSpec, ImageBufAlgo
 
@@ -34,6 +35,7 @@ try:
     checker = ImageBuf(ImageSpec(256, 256, 3, oiio.UINT8))
     ImageBufAlgo.checker (checker, 8, 8, 8, (0,0,0), (1,1,1))
     gray128 = make_constimage (128, 128, 3, oiio.HALF, (0.5,0.5,0.5))
+    gray64 = make_constimage (64, 64, 3, oiio.HALF, (0.5,0.5,0.5))
 
     # black
     b = ImageBuf (ImageSpec(320,240,3,oiio.UINT8))
@@ -51,6 +53,24 @@ try:
     ImageBufAlgo.checker (b, 64, 64, 64, (1,.5,.5), (.5,1,.5), 10, 5)
     write (b, "checker.tif")
 
+    # noise-uniform
+    b = ImageBuf (ImageSpec(64,64,3,oiio.UINT8))
+    ImageBufAlgo.zero (b)
+    ImageBufAlgo.noise (b, "uniform", 0.25, 0.75)
+    write (b, "noise-uniform3.tif")
+
+    # noise-gaussian
+    b = ImageBuf (ImageSpec(64,64,3,oiio.UINT8))
+    ImageBufAlgo.zero (b)
+    ImageBufAlgo.noise (b, "gaussian", 0.5, 0.1);
+    write (b, "noise-gauss.tif")
+
+    # noise-gaussian
+    b = ImageBuf (ImageSpec(64,64,3,oiio.UINT8))
+    ImageBufAlgo.zero (b)
+    ImageBufAlgo.noise (b, "salt", 1, 0.01);
+    write (b, "noise-salt.tif")
+
     # channels, channel_append
     b = ImageBuf()
     ImageBufAlgo.channels (b, grid, (0.25,2,"G"))
@@ -70,9 +90,14 @@ try:
 
     # flatten
     b = ImageBuf()
-    ImageBufAlgo.flatten (b, ImageBuf("../oiiotool/src/deepalpha.exr"))
+    ImageBufAlgo.flatten (b, ImageBuf("../oiiotool-deep/src/deepalpha.exr"))
     write (b, "flat.exr")
 
+    # deepen
+    b = ImageBuf()
+    ImageBufAlgo.deepen (b, ImageBuf("../oiiotool-deep/az.exr"))
+    write (b, "deepen.exr")
+
     # crop
     b = ImageBuf()
     ImageBufAlgo.crop (b, grid, oiio.ROI(50,150,200,600))
@@ -89,6 +114,21 @@ try:
     ImageBufAlgo.paste (b, 150, 75, 0, 0, grid)
     write (b, "pasted.tif")
 
+    # rotate90
+    b = ImageBuf()
+    ImageBufAlgo.rotate90 (b, ImageBuf("../oiiotool/image.tif"))
+    write (b, "rotate90.tif")
+
+    # rotate180
+    b = ImageBuf()
+    ImageBufAlgo.rotate180 (b, ImageBuf("../oiiotool/image.tif"))
+    write (b, "rotate180.tif")
+
+    # rotate270
+    b = ImageBuf()
+    ImageBufAlgo.rotate270 (b, ImageBuf("../oiiotool/image.tif"))
+    write (b, "rotate270.tif")
+
     # flip
     b = ImageBuf()
     ImageBufAlgo.flip (b, ImageBuf("../oiiotool/image.tif"))
@@ -104,6 +144,16 @@ try:
     ImageBufAlgo.flipflop (b, ImageBuf("../oiiotool/image.tif"))
     write (b, "flipflop.tif")
 
+    # reorient
+    b = ImageBuf()
+    image_small = ImageBuf()
+    ImageBufAlgo.resample (image_small, ImageBuf("../oiiotool/image.tif"),  roi=oiio.ROI(0,160,0,120))
+    ImageBufAlgo.rotate90 (image_small, image_small)
+    image_small.specmod().attribute ("Orientation", 8)
+    ImageBufAlgo.reorient (b, image_small)
+    write (b, "reorient1.tif")
+    image_small = ImageBuf()
+
     # transpose
     b = ImageBuf()
     ImageBufAlgo.transpose (b, ImageBuf("../oiiotool/image.tif"))
@@ -139,6 +189,19 @@ try:
                       make_constimage(64,64,3,oiio.HALF,(.1,.1,.1),20,20))
     write (b, "sub.exr")
 
+    # Test --absdiff and --abs
+    # First, make a test image that's 0.5 on the left, -0.5 on the right
+    a = ImageBuf (ImageSpec(128,128,3,oiio.HALF))
+    ImageBufAlgo.fill (a, (0.5,0.5,0.5))
+    ImageBufAlgo.fill (a, (-0.25,-0.25,-0.25), oiio.ROI(0,64,0,128))
+    b = ImageBuf()
+    ImageBufAlgo.abs (b, a)
+    write (b, "abs.exr", oiio.HALF)
+    b = ImageBuf()
+    ImageBufAlgo.absdiff (b, a, (0.2,0.2,0.2))
+    write (b, "absdiff.exr", oiio.HALF)
+    a = ImageBuf()
+
     # mul
     b = ImageBuf()
     ImageBufAlgo.mul (b, gray128, 1.5)
@@ -146,11 +209,34 @@ try:
     b = ImageBuf()
     ImageBufAlgo.mul (b, gray128, (1.5,1,0.5))
     write (b, "cmul2.exr")
-    # FIXME -- image multiplication; it's not in testsuite/oiiotool either
-    # b = ImageBuf()
-    # ImageBufAlgo.mul (b, make_constimage(64,64,3,oiio.HALF,(.1,.2,.3)),
-    #                        make_constimage(64,64,3,oiio.HALF,(.1,.1,.1),20,20))
-    # write (b, "mul.exr")
+    b = ImageBuf()
+    ImageBufAlgo.mul (b, make_constimage(64,64,3,oiio.HALF,(.5,.5,.5)),
+                         make_constimage(64,64,3,oiio.HALF,(1.5,1,0.5)))
+    write (b, "mul.exr", oiio.HALF)
+
+    # mad
+    b = ImageBuf()
+    ImageBufAlgo.mad (b, make_constimage(64,64,3,oiio.HALF,(.5,.5,.5)),
+                         make_constimage(64,64,3,oiio.HALF,(1.5,1,0.5)),
+                         make_constimage(64,64,3,oiio.HALF,(0.1,0.1,0.1)))
+    write (b, "mad.exr", oiio.HALF)
+
+    # div
+    b = ImageBuf()
+    ImageBufAlgo.div (b, gray64, make_constimage (64, 64, 3, oiio.HALF, (2.0,1,0.5)))
+    write (b, "div.exr", oiio.HALF)
+    b = ImageBuf()
+    ImageBufAlgo.div (b, gray64, 2.0)
+    write (b, "divc1.exr", oiio.HALF)
+    b = ImageBuf()
+    ImageBufAlgo.div (b, gray64, (2.0,1,0.5))
+    write (b, "divc2.exr", oiio.HALF)
+
+    # invert
+    a = ImageBuf ("../oiiotool/tahoe-small.tif")
+    b = ImageBuf()
+    ImageBufAlgo.invert (b, a)
+    write (b, "invert.tif", oiio.UINT8)
 
     # pow
     b = ImageBuf()
@@ -229,23 +315,49 @@ try:
     ImageBufAlgo.resample (b, grid, roi=oiio.ROI(0,128,0,128))
     write (b, "resample.tif")
 
+    # warp
+    b = ImageBuf()
+    Mwarp = (0.7071068, 0.7071068, 0, -0.7071068, 0.7071068, 0, 128, -53.01933, 1)
+    ImageBufAlgo.warp (b, ImageBuf("resize.tif"), Mwarp)
+    write (b, "warped.tif")
+
+    # rotate
+    b = ImageBuf()
+    ImageBufAlgo.rotate (b, ImageBuf("resize.tif"), math.radians(45.0))
+    write (b, "rotated.tif")
+    b = ImageBuf()
+    ImageBufAlgo.rotate (b, ImageBuf("resize.tif"), math.radians(45.0), 50.0, 50.0)
+    write (b, "rotated-offcenter.tif")
+
     # make_kernel
     bsplinekernel = ImageBuf()
     ImageBufAlgo.make_kernel (bsplinekernel, "bspline", 15, 15)
     write (bsplinekernel, "bsplinekernel.exr")
 
-    # convolve
+    # convolve -- test with bspline blur
     b = ImageBuf()
     ImageBufAlgo.convolve (b, ImageBuf("../oiiotool/tahoe-small.tif"),
                            bsplinekernel)
     write (b, "bspline-blur.tif", oiio.UINT8)
 
+    # median filter
+    b = ImageBuf()
+    ImageBufAlgo.median_filter (b, ImageBuf("../oiiotool/tahoe-small.tif"),
+                                5, 5)
+    write (b, "tahoe-median.tif", oiio.UINT8)
+
     # unsharp_mask
     b = ImageBuf()
     ImageBufAlgo.unsharp_mask (b, ImageBuf("../oiiotool/tahoe-small.tif"),
                                "gaussian", 3.0, 1.0, 0.0)
     write (b, "unsharp.tif", oiio.UINT8)
 
+    # unsharp_mark with median filter
+    b = ImageBuf()
+    ImageBufAlgo.unsharp_mask (b, ImageBuf("../oiiotool/tahoe-small.tif"),
+                               "median", 3.0, 1.0, 0.0)
+    write (b, "unsharp-median.tif", oiio.UINT8)
+
     # computePixelHashSHA1
     print ("SHA-1 of bsplinekernel.exr is: " + 
            ImageBufAlgo.computePixelHashSHA1(bsplinekernel))
@@ -253,15 +365,15 @@ try:
     # fft, ifft
     fft = ImageBuf()
     blue = ImageBuf()
-    ImageBufAlgo.channels (blue, ImageBuf("../oiiotool/tahoe-small.tif"),
+    ImageBufAlgo.channels (blue, ImageBuf("../oiiotool/tahoe-tiny.tif"),
                            (2,))
     ImageBufAlgo.fft (fft, blue)
-    write (fft, "fft.exr", oiio.HALF)
+    write (fft, "fft.exr", oiio.FLOAT)
     inv = ImageBuf()
     ImageBufAlgo.ifft (inv, fft)
     b = ImageBuf()
     ImageBufAlgo.channels (b, inv, (0,0,0))
-    write (b, "ifft.exr", oiio.HALF)
+    write (b, "ifft.exr", oiio.FLOAT)
     inv.clear()
     fft.clear()
 
@@ -270,8 +382,8 @@ try:
     ImageBufAlgo.complex_to_polar (polar, fft)
     b = ImageBuf()
     ImageBufAlgo.polar_to_complex (b, polar)
-    write (polar, "polar.exr", oiio.HALF)
-    write (b, "complex.exr", oiio.HALF)
+    write (polar, "polar.exr", oiio.FLOAT)
+    write (b, "unpolar.exr", oiio.FLOAT)
     fft.clear()
     polar.clear()
 
@@ -295,7 +407,12 @@ try:
 
     # FIXME - no test for zover (not in oiio-composite either)
 
-    # FIXME - no test for render_text
+    b = make_constimage (320, 240, 3, oiio.FLOAT)
+    ImageBufAlgo.render_text (b, 25, 50, "Hello, world",
+                              16, "DroidSerif", (1,1,1))
+    ImageBufAlgo.render_text (b, 50, 120, "Go Big Red!",
+                              42, "", (1,0,0))
+    write (b, "text.tif", oiio.UINT8)
 
     # histogram, histogram_draw,
 
